const fs = require('fs')
const path = require('path')
const i18NPlugin = require('../src/index.js')
const pluginTester = require('babel-plugin-tester')

const pluginOptions = {
    language: 'de',
    inputPath: '__tests__/i18n',
    outputPath: '__tests__/i18n'
}

const mockLanguageFileContent = [
    {},
    {
        Hi: 'Hi// TODO',
        Hallo: 'Hallo// TODO',
        Hello: 'Hello// TODO'
    },
    {
        Hi: 'Hi// TODO',
        Hallo: 'Hallo// TODO'
    }
]

const mockJavascriptFileContent = [
    {
        code: 'var hello = __("Hello"); var hi = __("Hi"); var hallo = __("Hallo");',
        parsedValue: {
            Hello: 'Hello// TODO',
            Hi: 'Hi// TODO',
            Hallo: 'Hallo// TODO'
        }
    },
    {
        code: 'var hello = __("Hello"); var hi = __("Hi"); var hallo = __("Hallo");',
        parsedValue: {
            Hello: 'Hello// TODO',
            Hi: 'Hi// TODO',
            Hallo: 'Hallo// TODO'
        }
    },
    {
        code: 'var hello = __("Hello"); var newValue = __("newValue"); var hallo = __("Hallo");',
        parsedValue: {
            Hello: 'Hello// TODO',
            newValue: 'newValue// TODO',
            Hallo: 'Hallo// TODO'
        }
    }
]

function setupReadLanguageFile() {
    try {
        const fileLocation = `${process.cwd()}/${pluginOptions.inputPath}/${pluginOptions.language}.json`
        const fileContent = JSON.parse(fs.readFileSync(fileLocation))

        return fileContent
    } catch (error) {
        throw new Error('Not able to read the language file.')
    }
}

function writeToLanguageFile(content) {
    try {
        const outputLocation = `${process.cwd()}/${pluginOptions.inputPath}/${
            pluginOptions.language
        }.json`
        fs.writeFileSync(outputLocation, JSON.stringify(content, null, 4))
    } catch (err) {
        throw new Error('File should have been created')
    }
}

function verifyLanguageFile(content) {
    let fileContent = null

    try {
        const fileLocation = `${process.cwd()}/${pluginOptions.inputPath}/${pluginOptions.language}.json`
        fileContent = JSON.parse(fs.readFileSync(fileLocation))
    } catch (error) {
        throw new Error('Not able to open and compare language file.')
    }
    if (fileContent) {
        Object.keys(fileContent).forEach(key => {
            const languageValue = content[key]
            expect(languageValue).toBe(fileContent[key])
        })
    }
}

function removeLanguageFile() {
    try {
        fs.unlinkSync(`${process.cwd()}/${pluginOptions.inputPath}/${pluginOptions.language}.json`)
    } catch (error) {}
}

pluginTester({
    plugin: i18NPlugin,
    snapshot: true,
    pluginOptions,
    tests: [
        {
            title: 'Language File should be created if it does not exist',
            setup() {
                removeLanguageFile()
                expect(setupReadLanguageFile).toThrow('Not able to read the language file.')
            },
            code: mockJavascriptFileContent[0].code,
            teardown() {
                expect(setupReadLanguageFile).toBeDefined()
                removeLanguageFile()
            }
        },
        {
            title: 'Language File should exists and keep the same content',
            setup() {
                writeToLanguageFile(mockLanguageFileContent[2])
            },
            code: mockJavascriptFileContent[1].code,
            teardown() {
                verifyLanguageFile(mockJavascriptFileContent[1].parsedValue)
                removeLanguageFile()
            }
        },
        {
            title: 'Language File should exists and change the current content',
            setup() {
                writeToLanguageFile(mockLanguageFileContent[2])
            },
            pluginOptions: {
                ...pluginOptions,
                test: true
            },
            code: mockJavascriptFileContent[2].code,
            teardown() {
                verifyLanguageFile(mockJavascriptFileContent[2].parsedValue)
                removeLanguageFile()
            }
        }
    ]
})
